<?php /** @noinspection PhpUndefinedMethodInspection */
/** @noinspection PhpParamsInspection */
declare(strict_types=1);

namespace IntiTest\App;

use Inti\App\ConfigProvider;
use Inti\Test\TestCase\ApplicationTestCase;
use Mezzio\Application;
use PHPUnit\Framework\TestCase;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Class ConfigProviderTest
 *
 * @package IntiTest\App
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-app
 */
final class ConfigProviderTest extends TestCase
{
    use ApplicationTestCase;
    use ProphecyTrait;

    protected array $providers = [
        ConfigProvider::class,
    ];

    protected function setUp(): void
    {
        $this->setUpContainer();
    }

    public function testCanCreateTheApplication()
    {
        $this->assertServiceInstanceOf(Application::class, Application::class);
    }

    public function testCanPipeMiddlewareViaConfig()
    {
        /** @var MiddlewareInterface|ObjectProphecy $middleware */
        $middleware = $this->prophesize(MiddlewareInterface::class);
        $middleware->process(Argument::any(), Argument::any())->shouldBeCalled();

        $this->runApplication([
            'middleware_pipeline' => [[
                'middleware' => $middleware->reveal(),
                'priority' => 50,
            ]],
        ]);
    }

    public function testCanRouteHandlerViaConfig()
    {
        /** @var RequestHandlerInterface|ObjectProphecy $handler */
        $handler = $this->prophesize(RequestHandlerInterface::class);
        $handler->handle(Argument::any())->shouldBeCalled();

        $this->runApplication([
            'routes' => [[
                'path' => '/test-route',
                'middleware' => $handler->reveal(),
            ]]
        ], '/test-route');
    }
}
